@@ -46,6 +46,7 @@
 #endif
 
 #include <QDBusError>
+#include <QDBusMetaType>
 #include <QDBusPendingReply>
 #include <QIcon>
 
@@ -165,6 +166,56 @@ void Handler::activateConnection(const QString& connection, const QString& devic
     connect(watcher, &QDBusPendingCallWatcher::finished, this, &Handler::replyFinished);
 }
 
+QString Handler::wifiCode(const QString& connectionPath, const QString& ssid, int _securityType) const
+{
+    NetworkManager::WirelessSecurityType securityType = static_cast<NetworkManager::WirelessSecurityType>(_securityType);
+
+    QString ret = QStringLiteral("WIFI:S:") + ssid + QLatin1Char(';');
+    if (securityType != NetworkManager::NoneSecurity) {
+        switch (securityType) {
+            case NetworkManager::NoneSecurity:
+                break;
+            case NetworkManager::StaticWep:
+                ret += "T:WEP;";
+                break;
+            case NetworkManager::WpaPsk:
+            case NetworkManager::Wpa2Psk:
+                ret += "T:WPA;";
+                break;
+            default:
+            case NetworkManager::DynamicWep:
+            case NetworkManager::WpaEap:
+            case NetworkManager::Wpa2Eap:
+            case NetworkManager::Leap:
+                return {};
+        }
+    }
+
+    NetworkManager::Connection::Ptr connection = NetworkManager::findConnection(connectionPath);
+    if(!connection)
+        return {};
+
+    const auto key = QStringLiteral("802-11-wireless-security");
+    auto reply = connection->secrets(key);
+
+    const auto secret = reply.argumentAt<0>()[key];
+    QString pass;
+    switch (securityType) {
+        case NetworkManager::NoneSecurity:
+            break;
+        case NetworkManager::WpaPsk:
+        case NetworkManager::Wpa2Psk:
+            pass = secret["psk"].toString();
+            break;
+        default:
+            return {};
+    }
+    if (!pass.isEmpty())
+        ret += QStringLiteral("P:") + pass + QLatin1Char(';');
+
+    return ret + QLatin1Char(';');
+}
+
 void Handler::addAndActivateConnection(const QString& device, const QString& specificObject, const QString& password)
 {
     NetworkManager::AccessPoint::Ptr ap;
@@ -350,8 +401,8 @@ void Handler::enableBluetooth(bool enable)
                         if (!enable) {
                             QDBusMessage message = QDBusMessage::createMethodCall("org.bluez", objPath, "org.freedesktop.DBus.Properties", "Get");
                             QList<QVariant> arguments;
-                            arguments << QLatin1Literal("org.bluez.Adapter1");
-                            arguments << QLatin1Literal("Powered");
+                            arguments << QLatin1String("org.bluez.Adapter1");
+                            arguments << QLatin1String("Powered");
                             message.setArguments(arguments);
                             QDBusPendingReply<QVariant> getReply = QDBusConnection::systemBus().asyncCall(message);
                             QDBusPendingCallWatcher *getWatcher = new QDBusPendingCallWatcher(getReply, this);
@@ -362,8 +413,8 @@ void Handler::enableBluetooth(bool enable)
                                             m_bluetoothAdapters.insert(objPath, reply.value().toBool());
                                             QDBusMessage message = QDBusMessage::createMethodCall("org.bluez", objPath, "org.freedesktop.DBus.Properties", "Set");
                                             QList<QVariant> arguments;
-                                            arguments << QLatin1Literal("org.bluez.Adapter1");
-                                            arguments << QLatin1Literal("Powered");
+                                            arguments << QLatin1String("org.bluez.Adapter1");
+                                            arguments << QLatin1String("Powered");
                                             arguments << QVariant::fromValue(QDBusVariant(QVariant(false)));
                                             message.setArguments(arguments);
                                             QDBusConnection::systemBus().asyncCall(message);
@@ -373,8 +424,8 @@ void Handler::enableBluetooth(bool enable)
                         } else if (enable && m_bluetoothAdapters.value(objPath)) {
                             QDBusMessage message = QDBusMessage::createMethodCall("org.bluez", objPath, "org.freedesktop.DBus.Properties", "Set");
                             QList<QVariant> arguments;
-                            arguments << QLatin1Literal("org.bluez.Adapter1");
-                            arguments << QLatin1Literal("Powered");
+                            arguments << QLatin1String("org.bluez.Adapter1");
+                            arguments << QLatin1String("Powered");
                             arguments << QVariant::fromValue(QDBusVariant(QVariant(enable)));
                             message.setArguments(arguments);
                             QDBusConnection::systemBus().asyncCall(message);
@@ -439,10 +490,12 @@ void Handler::requestScan(const QString &interface)
     for (NetworkManager::Device::Ptr device : NetworkManager::networkInterfaces()) {
         if (device->type() == NetworkManager::Device::Wifi) {
             NetworkManager::WirelessDevice::Ptr wifiDevice = device.objectCast<NetworkManager::WirelessDevice>();
-            if (wifiDevice) {
+
+            if (wifiDevice && wifiDevice->state() != NetworkManager::WirelessDevice::Unavailable) {
                 if (!interface.isEmpty() && interface != wifiDevice->interfaceName()) {
                     continue;
                 }
+
                 qCDebug(PLASMA_NM) << "Requesting wifi scan on device" << wifiDevice->interfaceName();
                 QDBusPendingReply<> reply = wifiDevice->requestScan();
                 QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(reply, this);
@@ -522,7 +575,7 @@ void Handler::replyFinished(QDBusPendingCallWatcher * watcher)
             case Handler::RequestScan:
             {
                 const QString interface = watcher->property("interface").toString();
-                qCDebug(PLASMA_NM) << "Wireless scan on" << interface << "failed:" << error;
+                qCWarning(PLASMA_NM) << "Wireless scan on" << interface << "failed:" << error;
                 scanRequestFailed(interface);
                 break;
             }
